import { api } from '@nice/client';
import { message, Form, Modal } from 'antd';
import React, { useState, useEffect } from 'react';
import { SearchBar } from './components/SearchBar';
import { StaffTable } from './components/StaffTable';
import { StaffModal } from './components/StaffModal';
import { Staff, PaginatedResponse } from './components/types';

const TestPage: React.FC = () => {
    // 状态定义
    const [searchText, setSearchText] = useState('');
    const [currentPage, setCurrentPage] = useState(1);
    const [isModalVisible, setIsModalVisible] = useState(false);
    const [editingStaff, setEditingStaff] = useState<Staff | null>(null);
    const [form] = Form.useForm();
    const pageSize = 10;

    // API 调用
    const { data, isLoading, refetch } = api.staff.findManyWithPagination.useQuery<PaginatedResponse>({
        page: currentPage,
        pageSize: pageSize,
        where: {
            deletedAt: null,
            OR: searchText ? [
                { username: { contains: searchText } },
                { showname: { contains: searchText } }
            ] : undefined
        },
        select: {
            id: true,
            username: true,
            showname: true,
            absent: true,
        }
    });

    // 删除方法
    const deleteMutation = api.staff.softDeleteByIds.useMutation({
        onSuccess: () => {
            message.success('删除成功');
            refetch();
        },
        onError: (error) => {
            message.error('删除失败：' + error.message);
        }
    });

    // 更新方法
    const updateMutation = api.staff.update.useMutation({
        onSuccess: () => {
            message.success('更新成功');
            setIsModalVisible(false);
            refetch();
        }
    });

    // 创建方法
    const createMutation = api.staff.create.useMutation({
        onSuccess: () => {
            message.success('创建成功');
            setIsModalVisible(false);
            refetch();
        }
    });

    // 处理函数
    const handleSearch = () => {
        setCurrentPage(1);
        refetch();
    };

    const handleAdd = () => {
        setEditingStaff(null);
        form.resetFields();
        setIsModalVisible(true);
    };

    const handleEdit = (staff: Staff) => {
        setEditingStaff(staff);
        form.setFieldsValue(staff);
        setIsModalVisible(true);
    };

    const handleDelete = (id: string) => {
        Modal.confirm({
            title: '确认删除',
            content: '确定要删除这条记录吗？',
            onOk: async () => {
                try {
                    await deleteMutation.mutateAsync({ ids: [id] });
                } catch (error) {
                    console.error('删除失败:', error);
                }
            }
        });
    };

    const handlePageChange = (page: number) => {
        setCurrentPage(page);
        refetch(); // 确保切换页面时重新获取数据
    };

    const handleModalOk = async () => {
        try {
            const values = await form.validateFields();
            if (editingStaff) {
                await updateMutation.mutateAsync({
                    where: { id: editingStaff.id },
                    data: values
                });
            } else {
                await createMutation.mutateAsync({
                    data: values
                });
            }
        } catch (error) {
            message.error('操作失败：' + (error as Error).message);
        }
    };

    const handleImportSuccess = () => {
        refetch();
        message.success('数据已更新');
    };

    // 初始加载
    useEffect(() => {
        refetch();
    }, []);

    return (
        <div className="h-screen w-full p-6">  {/* 修改这里，使用 h-screen 而不是 h-full */}
            <h1 className="text-2xl font-bold mb-4 text-center">培训情况记录</h1>
            <SearchBar
                searchText={searchText}
                onSearchTextChange={setSearchText}
                onSearch={handleSearch}
                onAdd={handleAdd}
                onImportSuccess={handleImportSuccess}
                data={data?.items || []}
            />
            <StaffTable
                data={data?.items || []}
                total={data?.total || 0}
                currentPage={currentPage}
                pageSize={pageSize}
                isLoading={isLoading}
                onPageChange={handlePageChange}
                onEdit={handleEdit}
                onDelete={handleDelete}
            />
            <StaffModal
                visible={isModalVisible}
                editingStaff={editingStaff}
                form={form}
                onOk={handleModalOk}
                onCancel={() => setIsModalVisible(false)}
            />
        </div>
    );
};

export default TestPage;