import { Button, Upload, message } from 'antd';
import { UploadOutlined, DownloadOutlined } from '@ant-design/icons';
import React from 'react';
import { Staff } from './types';
import * as XLSX from 'xlsx';
import { api } from '@nice/client';

interface ImportExportButtonsProps {
  onImportSuccess: () => void;
  data: Staff[];
}

export const ImportExportButtons: React.FC<ImportExportButtonsProps> = ({
  onImportSuccess,
  data,
}) => {
  const createMutation = api.staff.create.useMutation({
    onSuccess: () => {
      // 成功时不显示单条消息，让最终统计来显示
    },
    onError: (error) => {
      // 只有当不是用户名重复错误时才显示错误信息
      if (!error.message.includes('Unique constraint failed')) {
        message.error('导入失败: ' + error.message);
      }
    }
  });

  // 添加恢复记录的 mutation
  const restoreMutation = api.staff.update.useMutation({
    onSuccess: () => {
      // 静默成功，不显示消息
    },
    onError: (error) => {
      console.error('恢复记录失败:', error);
    }
  });

  const handleImport = async (file: File) => {
    try {
      const reader = new FileReader();
      reader.onload = async (e) => {
        const workbook = XLSX.read(e.target?.result, { type: 'binary' });
        const sheetName = workbook.SheetNames[0];
        const worksheet = workbook.Sheets[sheetName];
        const jsonData = XLSX.utils.sheet_to_json(worksheet);

        let successCount = 0;
        let restoredCount = 0;
        let errorCount = 0;

        // 转换数据格式
        const staffData = jsonData.map((row: any) => ({
          username: row['用户名'],
          showname: row['名称'],
          absent: row['是否在位'] === '在位',
        }));

        // 批量处理
        for (const staff of staffData) {
          try {
            // 先尝试创建新记录
            await createMutation.mutateAsync({
              data: staff
            });
            successCount++;
          } catch (error: any) {
            // 如果是用户名重复错误
            if (error.message.includes('Unique constraint failed')) {
              try {
                // 尝试恢复已删除的记录
                await restoreMutation.mutateAsync({
                  where: {
                    username: staff.username,
                  },
                  data: {
                    deletedAt: null,
                    showname: staff.showname,
                    absent: staff.absent,
                  }
                });
                restoredCount++;
              } catch (restoreError) {
                errorCount++;
                console.error('恢复记录失败:', restoreError);
              }
            } else {
              errorCount++;
              console.error('创建记录失败:', error);
            }
          }
        }

        // 显示导入结果
        if (successCount > 0 || restoredCount > 0) {
          let successMessage = [];
          if (successCount > 0) {
            successMessage.push(`新增 ${successCount} 条`);
          }
          if (restoredCount > 0) {
            successMessage.push(`恢复 ${restoredCount} 条`);
          }
          message.success(`导入完成：${successMessage.join('，')}`);
          onImportSuccess();
        }
        if (errorCount > 0) {
          message.warning(`${errorCount} 条记录导入失败`);
        }
      };
      reader.readAsBinaryString(file);
    } catch (error) {
      message.error('文件读取失败');
    }
    return false;
  };

  const handleExport = () => {
    try {
      const exportData = data.map(staff => ({
        '用户名': staff.username,
        '名称': staff.showname,
        '是否在位': staff.absent ? '是' : '否'
      }));

      const worksheet = XLSX.utils.json_to_sheet(exportData);
      const workbook = XLSX.utils.book_new();
      XLSX.utils.book_append_sheet(workbook, worksheet, '员工列表');
      
      XLSX.writeFile(workbook, `员工列表_${new Date().toLocaleDateString()}.xlsx`);
      message.success('导出成功');
    } catch (error) {
      message.error('导出失败: ' + (error as Error).message);
    }
  };

  return (
    <div className="flex space-x-2">
      <Upload
        accept=".xlsx,.xls"
        showUploadList={false}
        beforeUpload={handleImport}
      >
        <Button icon={<UploadOutlined />}>导入</Button>
      </Upload>
      <Button
        icon={<DownloadOutlined />}
        onClick={handleExport}
      >
        导出
      </Button>
    </div>
  );
};