import { Button, Space, Table, Tag, Input, Pagination, message } from 'antd';
import { EditOutlined, DeleteOutlined } from '@ant-design/icons';
import React, { useState } from 'react';
import { Staff } from './types';


interface StaffTableProps {
    data: Staff[];
    total: number;
    currentPage: number;
    pageSize: number;
    isLoading?: boolean;
    onPageChange: (page: number) => void;
    onEdit: (staff: Staff) => void;
    onDelete: (id: string) => void;
}

export const StaffTable: React.FC<StaffTableProps> = ({
    data,
    total,
    currentPage,
    pageSize,
    isLoading = false,
    onPageChange,
    onEdit,
    onDelete,
}) => {
    const [jumpPage, setJumpPage] = useState('');

    const handleJumpPage = () => {
        const page = parseInt(jumpPage);
        if (!isNaN(page) && page > 0 && page <= Math.ceil(total / pageSize)) {
            onPageChange(page);
            setJumpPage('');
        } else {
            message.error('请输入有效的页码');
        }
    };

    const columns = [
        {
            title: '用户名',
            dataIndex: 'username',
            key: 'username',
        },
        {
            title: '名称',
            dataIndex: 'showname',
            key: 'showname',
        },
        {
            title: '是否在位',
            dataIndex: 'absent',
            key: 'absent',
            render: (absent: boolean) => (
                <Tag color={absent ? 'success' : 'error'}>
                    {absent ? '在位' : '不在位'}
                </Tag>
            ),
        },
        {
            title: '操作',
            key: 'action',
            render: (_: any, record: Staff) => (
                <Space size="middle">
                    <Button
                        type="link"
                        icon={<EditOutlined />}
                        onClick={() => onEdit(record)}
                    >
                        编辑
                    </Button>
                    <Button
                        type="link"
                        icon={<DeleteOutlined />}
                        danger
                        onClick={() => onDelete(record.id)}
                    >
                        删除
                    </Button>
                </Space>
            ),
        },
    ];

    // 修改样式定义
    const containerStyle = {
        display: 'flex',
        flexDirection: 'column' as const,
        height: '100%'
    };

    const tableContainerStyle = {
        flex: 1,
        height: 'calc(100vh - 240px)',  // 减去头部、搜索栏和分页的高度
        overflow: 'hidden'
    };

    const tableStyle = {
        height: '100%'
    };

    return (
        <div style={containerStyle}>
            <div style={tableContainerStyle}>
                <Table
                    dataSource={data}
                    columns={columns}
                    rowKey="id"
                    pagination={false}
                    loading={isLoading}
                    style={tableStyle}
                />
            </div>
            <div className="mt-6 flex justify-center items-center">
                <div className="bg-white px-6 py-3 rounded-lg shadow-sm flex items-center space-x-6">
                    <Pagination
                        current={currentPage}
                        total={total}
                        pageSize={pageSize}
                        onChange={onPageChange}
                        showTotal={(total) => (
                            <span className="text-gray-600">
                                共 <span className="font-medium text-gray-900">{total}</span> 条记录
                            </span>
                        )}
                        showSizeChanger={false}
                    />
                    <div className="h-6 w-px bg-gray-200" />
                    <div className="flex items-center space-x-2">
                        <Input
                            size="small"
                            style={{ width: 60 }}
                            value={jumpPage}
                            onChange={(e) => setJumpPage(e.target.value)}
                            onPressEnter={handleJumpPage}
                            placeholder="页码"
                            className="text-center"
                        />
                        <Button
                            size="small"
                            type="primary"
                            onClick={handleJumpPage}
                        >
                            跳转
                        </Button>
                    </div>
                </div>
            </div>
        </div>
    );
};