import React from "react";
import { Row, Col, Table, Badge, Empty, Tooltip } from "antd";
import { TeamOutlined, InfoCircleOutlined } from "@ant-design/icons";
import DashboardCard from "../../../components/presentation/dashboard-card";
import { theme } from "antd";
import { EChartsOption } from "echarts-for-react";

interface DeptData {
  id: string;
  name: string;
  count: number;
}
interface DepartmentTableProps {
  filteredDeptData: DeptData[];
  staffs: any[] | undefined;
}
// 图表配置函数
export const getPieChartOptions = (
  deptData: DeptData[],
  onEvents?: Record<string, (params: any) => void>
): EChartsOption => {
  const top10Depts = deptData.slice(0, 10);
  return {
    tooltip: {
      trigger: "item",
      formatter: "{a} <br/>{b}: {c}人 ({d}%)",
    },
    legend: {
      orient: "vertical",
      right: 10,
      top: "center",
      data: top10Depts.map((dept) => dept.name),
    },
    series: [
      {
        name: "部门人数",
        type: "pie",
        radius: ["50%", "70%"],
        avoidLabelOverlap: false,
        itemStyle: {
          borderRadius: 10,
          borderColor: "#fff",
          borderWidth: 2,
        },
        label: {
          show: true,
          formatter: "{b}: {c}人 ({d}%)",
          position: "outside",
        },
        emphasis: {
          label: {
            show: true,
            fontSize: "18",
            fontWeight: "bold",
          },
        },
        labelLine: {
          show: true,
        },
        data: top10Depts.map((dept) => ({
          value: dept.count,
          name: dept.name,
        })),
      },
    ],
  };
};

export const getBarChartOptions = (deptData: DeptData[]): EChartsOption => {
  const top10Depts = deptData.slice(0, 10);
  return {
    tooltip: {
      trigger: "axis",
      axisPointer: {
        type: "shadow",
      },
    },
    grid: {
      left: "3%",
      right: "12%",
      bottom: "3%",
      containLabel: true,
    },
    xAxis: {
      type: "value",
      boundaryGap: [0, 0.01],
    },
    yAxis: {
      type: "category",
      data: top10Depts.map((dept) => dept.name),
      inverse: true,
    },
    series: [
      {
        name: "人员数量",
        type: "bar",
        data: top10Depts.map((dept) => dept.count),
        itemStyle: {
          color: function (params) {
            const colorList = [
              "#91cc75",
              "#5470c6",
              "#ee6666",
              "#73c0de",
              "#3ba272",
              "#fc8452",
              "#9a60b4",
            ];
            return colorList[params.dataIndex % colorList.length];
          },
        },
        label: {
          show: true,
          position: "right",
          formatter: "{c}人",
        },
      },
    ],
  };
};

export default function DepartmentTable({
  filteredDeptData,
  staffs,
}: DepartmentTableProps): React.ReactElement {
  const { token } = theme.useToken();

  const deptColumns = [
    {
      title: "部门名称",
      dataIndex: "name",
      key: "name",
    },
    {
      title: "人员数量",
      dataIndex: "count",
      key: "count",
      render: (count: number) => (
        <Badge
          count={count}
          overflowCount={999}
          style={{ backgroundColor: token.colorPrimary }}
        />
      ),
    },
    {
      title: "占比",
      dataIndex: "count",
      key: "percentage",
      render: (count: number) => (
        <span>
          {staffs && staffs.length > 0
            ? ((count / staffs.length) * 100).toFixed(1) + "%"
            : "0%"}
        </span>
      ),
    },
  ];

  return (
    <Row className="mt-4">
      <Col span={24}>
        <DashboardCard
          title={
            <div className="flex items-center justify-between w-full">
              <div>
                <TeamOutlined className="mr-2" />
                <span>部门人员分布详情</span>
              </div>
              <Tooltip title="点击饼图可筛选特定部门数据">
                <InfoCircleOutlined className="text-gray-400" />
              </Tooltip>
            </div>
          }
          className="min-h-80"
        >
          {filteredDeptData.length > 0 ? (
            <Table
              dataSource={filteredDeptData}
              columns={deptColumns}
              rowKey="id"
              pagination={{
                pageSize: 10,
                showTotal: (total) => `共 ${total} 个部门`,
              }}
              size="small"
              className="mt-2"
            />
          ) : (
            <Empty description="暂无部门数据" className="my-8" />
          )}
        </DashboardCard>
      </Col>
    </Row>
  );
}
